using System;
using Microsoft.SqlServer.Dts.Pipeline;
using Microsoft.SqlServer.Dts.Runtime;
using Microsoft.SqlServer.Dts.Pipeline.Wrapper;
using Microsoft.SqlServer.Dts.Runtime.Wrapper;

namespace SSIS2K8.Samples.Transforms.ImageFileSrc
{
	/// <summary>
	/// Reads EXIF information from jpg files.
	/// </summary>
	[ DtsPipelineComponent(
		ComponentType = ComponentType.SourceAdapter,
		IconResource = "SSIS2K8.Samples.Transforms.ImageFileSrc.component.ico", 
      DisplayName="ImageFile Source",
      Description="Retrieves EXIF information from JPG files",      
		CurrentVersion = 1) ]   
	public class ImageFileSrc : Microsoft.SqlServer.Dts.Pipeline.PipelineComponent
	{
      internal struct ColumnInfo
      {
         // Constructor
         public ColumnInfo(string n, DataType t, int l)
         {
            name = n;
            type = t;
            length = l;
            outputExists = false;
            outputID = -1;
         }

         // Public Members
         public string name;
         public DataType type;
         public int length;
         public bool outputExists;
         public int outputID;
      };
      

		#region Private constants
		private const string CONNECTION_NAME = "MultiFileConnection";
      private const string CONNECTION_DESC = "The image files with EXIF information";
      
      #endregion

      #region Private members
      private ColumnInfo[] m_columnInfos;
      IDTSConnectionManager100 m_cm;
		#endregion

		#region Constructor
      public ImageFileSrc()
      {
         m_columnInfos = new ColumnInfo[12]
         {
            new ColumnInfo("FILENAME", DataType.DT_STR, 1028),
            new ColumnInfo("MAKE", DataType.DT_STR, 30),
            new ColumnInfo("MODEL", DataType.DT_STR, 30),
            new ColumnInfo("SHUTTERSPEED", DataType.DT_STR, 10),
            new ColumnInfo("FSTOP", DataType.DT_STR, 10),
            new ColumnInfo("EXPOSUREPROGRAM", DataType.DT_STR, 20),
            new ColumnInfo("ISO", DataType.DT_STR, 10),
            new ColumnInfo("DATETIME", DataType.DT_STR, 40),
            new ColumnInfo("IMAGEWIDTH", DataType.DT_STR, 10),
            new ColumnInfo("IMAGELENGTH", DataType.DT_STR, 10),
            new ColumnInfo("BITSPERSAMPLE", DataType.DT_STR, 10),
            new ColumnInfo("COMPRESSION", DataType.DT_STR, 20),
         };


      }
		#endregion

		#region IDTSDesigntimeComponent100 methods

      // Called multiple times during design and once during execution
		public override void AcquireConnections(object transaction)
		{
			IDTSRuntimeConnection100 conn;         
			try
			{          
				// get the runtime connection
				conn = ComponentMetaData.RuntimeConnectionCollection[CONNECTION_NAME];            
			}
			catch (Exception ex)
			{
				bool bCancel;
				ComponentMetaData.FireError(1, "ImageFileSrc", "Could not find the runtime connection.", "", 0, out bCancel );					
				throw ex;
			}
                  
			// Get the connection manager host from the connection
			ConnectionManagerHost cmh = (ConnectionManagerHost)conn.ConnectionManager;
                  
			if (cmh == null)
			{
				bool bCancel;
				ComponentMetaData.FireError(2, "ImageFileSrc", "Could not get the runtime Connection Manager.", "", 0, out bCancel );					
			}
         // Make sure it is a multifile connection manager
         else if ( cmh.CreationName != "MULTIFILE")
         {
            bool bCancel;
            ComponentMetaData.FireError(2, "ImageFileSrc", "Connection manager must be of type MULTIFILE", "", 0, out bCancel);
         }
         else
         {

            try
            {
               // Store the connection manager for validation and execution
               m_cm = conn.ConnectionManager;
            }
            catch (Exception ex)
            {
               bool bCancel;
               ComponentMetaData.FireError(10, "ImageFileSrc", "Could not get the runtime connection.", "", 0, out bCancel);
               throw ex;
            }           
         }
		}

      // Release if you have one, multifile only returns strings
		public override void ReleaseConnections()
		{
         // Nothing to do here since we're using file connection managers
         base.ReleaseConnections();         
		}
 
      // Describe what the component needs from the system and any custom properties.
		public override void ProvideComponentProperties()
		{
			// start out clean, remove anything put on by the base class
			RemoveAllInputsOutputsAndCustomProperties();

			// Get the runtime connection collection.
			IDTSRuntimeConnectionCollection100 pIDTSRuntimeConnectionCollection = ComponentMetaData.RuntimeConnectionCollection;

			// See if there	is already a runtime connection
			IDTSRuntimeConnection100 pIDTSRuntimeConnection;
			try
			{
				pIDTSRuntimeConnection = pIDTSRuntimeConnectionCollection[CONNECTION_NAME];
			}
			catch (Exception)
			{
				// must not be there, make one
				pIDTSRuntimeConnection = pIDTSRuntimeConnectionCollection.New();
				pIDTSRuntimeConnection.Name = CONNECTION_NAME;
				pIDTSRuntimeConnection.Description = CONNECTION_DESC;
			}

         //
         // Setup component output and columns
         //

			// Add an output
			IDTSOutput100 output = ComponentMetaData.OutputCollection.New();

         // Name the output
         output.Name = "Output";

         // Set external metadata used.
         ComponentMetaData.OutputCollection[0].ExternalMetadataColumnCollection.IsUsed = true;

         // We don't have connected metadata, just external metadata.
         // Don't provide connected validation.
         ComponentMetaData.ValidateExternalMetadata = false;

         // Don't care about error dispositions
         ComponentMetaData.UsesDispositions = true;
                  
         //
         // Setup component information
         //

         // Get the assembly version and set that as our current version.
         SetComponentVersion();

         // Name the component and add component information
         ComponentMetaData.Name = "ImageFileSrc";
         ComponentMetaData.Description = "Sample Image File Source Adapter";
         ComponentMetaData.ContactInfo = ComponentMetaData.Description +
            ";Microsoft SQL Server Integration Services 2005, By Kirk Haselden;" +
            " (c) 2006 Kirk Haselden" +
            ComponentMetaData.Version.ToString();         

		}

      // Called any time the metadata needs refreshing.
      // Called when Refresh button clicked on advanced UI
      // Called when validate returns VS_NEEDSNEWMETADATA
		public override void ReinitializeMetaData()
		{
			// Baseclass may have some work to do here
			base.ReinitializeMetaData();

         // Get the output
         IDTSOutput100 output = ComponentMetaData.OutputCollection[0];

         // Start from a clean slate
         output.OutputColumnCollection.RemoveAll();
         output.ExternalMetadataColumnCollection.RemoveAll();


         // Add the EXIF column information
         for (int i = 0; i < m_columnInfos.Length;i++ )
         {
            // Create a new output column
            IDTSOutputColumn100 column = output.OutputColumnCollection.NewAt(i);
            IDTSExternalMetadataColumn100 externalColumn = output.ExternalMetadataColumnCollection.NewAt(i);

            // Set the column properties
            column.SetDataTypeProperties(m_columnInfos[i].type, m_columnInfos[i].length, 0, 0, 1252);
            column.Name = m_columnInfos[i].name;

            // Set the external column properties
            externalColumn.DataType = m_columnInfos[i].type;
            externalColumn.Length = m_columnInfos[i].length;
            externalColumn.Name = m_columnInfos[i].name;

            // Assign the output column to the external metadata
            column.ExternalMetadataColumnID = externalColumn.ID;

            // Exclusion group
            output.ExclusionGroup = 0;

            // Synchronous input
            output.SynchronousInputID = 0;

            output.ErrorRowDisposition = DTSRowDisposition.RD_IgnoreFailure;
            output.TruncationRowDisposition = DTSRowDisposition.RD_IgnoreFailure;
            output.ErrorOrTruncationOperation = "Error or Truncation Operation !!!";

         }         
		}

		public override DTSValidationStatus Validate()
		{
         // Make sure base call is successful
			DTSValidationStatus status = base.Validate();
			if (status == DTSValidationStatus.VS_ISCORRUPT)
			{
				return status;
			}

         // If there are no external metadata columns, then return that we need some
         // Short circuit the validation since there is no metadata to validate at this point.
         if (ComponentMetaData.OutputCollection[0].ExternalMetadataColumnCollection.Count == 0)
            return DTSValidationStatus.VS_NEEDSNEWMETADATA;

			// should have no inputs
			if (ComponentMetaData.InputCollection.Count != 0)
				return DTSValidationStatus.VS_ISCORRUPT;

			// should have two outputs
			if (ComponentMetaData.OutputCollection.Count != 1)
				return DTSValidationStatus.VS_ISCORRUPT;


			// See if the UsesDispositions is set.
			if (!ComponentMetaData.UsesDispositions)
			{
				bool bCancel;
            ComponentMetaData.FireError( 99, "ImageFile Source", "Uses dispositions setting is incorrect", "", 0, out bCancel);
				return DTSValidationStatus.VS_ISCORRUPT;
			}

			return status;
		}
		#endregion

		#region IDTSRuntimeComponent100 methods
      // Called once right before executing.
		public override void PreExecute()
		{
         base.PreExecute();

         // Find out which output columns exist
         IDTSOutput100 output = ComponentMetaData.OutputCollection[0];
         int countColumns = output.OutputColumnCollection.Count;

         // Iterate through the columns
         for( int iColumn = 0; iColumn < countColumns; iColumn++ )
         {
            // Set the exists flag for the column info.
            m_columnInfos[iColumn].outputExists = 
               (output.OutputColumnCollection[iColumn] == null ? false : true);
            m_columnInfos[iColumn].outputID = iColumn;
         }
		}

      // Called to provide a buffer to fill. Only called once per execution.
      public override void PrimeOutput(int outputs, int[] outputIDs, PipelineBuffer[] buffers)
		{
         // Only have one output, so 
         PipelineBuffer buffer = buffers[0];

         // The name of the jpg file
         string fileName;

         // For each jpg file, get the EXIF data
         // MultiFile connection managers return null from 
         // AcquireConnection when the file list is exausted.
         object conn;
         while (null != (conn = m_cm.AcquireConnection(null)))
         {
            // Get the name of the file
            fileName = conn.ToString();

            // Open each file and read the EXIF information
            ExifReader reader = new ExifReader(fileName);

            // Add a new row to the buffer
            buffer.AddRow();

            // Insert each column into the buffer.
            foreach (ColumnInfo ci in m_columnInfos)
            {
               try
               {
                  // Only output to columns the user has chosen
                  if (ci.outputExists)
                  {
                     // We only have string types
                     // For other types, switch/case works.
                     buffer.SetString(ci.outputID, reader.GetProperty(ci.name));                     
                  }
               }
               catch (Exception ex)
               {
                  if (ex is DoesNotFitBufferException)
                  {
                     DTSRowDisposition rd = ComponentMetaData.OutputCollection[0].TruncationRowDisposition;
                     if (rd == DTSRowDisposition.RD_IgnoreFailure)
                     {
                        // Ignore the error and continue.
                        ComponentMetaData.FireWarning(100, "ImageFleSrc",
                           "The file property " + ci.name + " for file " + reader.FileName + " was too long. Ignoring truncation.", "", 0);
                     }
                     else                     
                     {
                        bool bCancel = false;
                        ComponentMetaData.FireError(100, "ImageFleSrc",
                           "The file property " + ci.name + " for file " + reader.FileName + " was too long.", "", 0, out bCancel);
                        throw ex;
                     }
                  } // if
               } // catch
            } // foreach
         } // while

         // Tell the buffer that we're done now.
         buffer.SetEndOfRowset();
      }
      #endregion

      #region Private methods
      /// <summary>
      /// Set the componet version to match the assembly version.
      /// </summary>
      public void SetComponentVersion()
		{
			// Get the assembly version and set that as our current version.
			DtsPipelineComponentAttribute attr = (DtsPipelineComponentAttribute)
					Attribute.GetCustomAttribute(this.GetType(), typeof(DtsPipelineComponentAttribute), false);
			ComponentMetaData.Version = attr.CurrentVersion;
		}
      #endregion
	}
}

